      PROGRAM odie
!-------------------------------------------------------------------------------
! General Description:
! ====================
!   ODIE  OTIS Data Interactive Examiner
!   A program to examine tabluar data input used in OTIS
!
!   ODIE is a product of the United States Government and is in the public domain.
!
! History of Code Changes:
! ========================
! J.P.Riehl    Added changes to run with the OTIS 4 directory structure, 
!              small bug fixes, new formatting and functionality,
!              and work with alternate pcal names. This removes the use 
!              of ODIE for detecting invalid ABLOCK names.
!              April, 2007
!
! J.P.Riehl    Removed the populating of arrays with ABLOCK names. 
!              Removed call to parser and associate linkage to ABLOCK location.
!              set start of alternate pcal names to the start of ABLOCK. 
!              May, 2007
!
! J.P.Riehl    Created an index (m) that points to the correct values of a 
!              variable's min and max values. Also added logic to prevent 
!              division by 0 when determining the number of points associated
!              with a particular variable. Added recognition of 0,0,0 to mean
!              the range is 0 to 0 with inc. of 0 instead of default min,
!              default max, increment of 0. Added logic in big plotting loop to
!              limit the occurances of blank records. Previously any variable with
!              a range of one point (i.e. a constant for the evaluation)
!              forced a blank record in the plot file.
!              March, 2008
!
! J.A. Huwaldt Added FLUSH statements before each interactive READ so that
!               the program can be used in a non-interactive (piped input/output)
!               manner.
!               November, 2013
!
! Author:      C R Hargraves
! Institution: Boeing Aerospace
! Project:     OTIS 1 Development, 1987.
!-------------------------------------------------------------------------------
      IMPLICIT NONE
!
      INCLUDE 'PAR1.CD'
      INCLUDE 'ABLOCK.CD'
      INCLUDE 'ABNBKC.CD'
      INCLUDE 'ABNBKN.CD'
      INCLUDE 'ABOBK.CD'
      INCLUDE 'AXNBKN.CD'
      INCLUDE 'DBCHAR.CD'
      INCLUDE 'DBCONT.CD'
      INCLUDE 'FINDC.CD'
      INCLUDE 'PATHS.CD'
      INCLUDE 'UNITS.CD'
!
      CHARACTER*132 aline
      CHARACTER*132 fmtdata
      CHARACTER*132 fmthead
      CHARACTER*16 dbn
      CHARACTER*16 dbname
      CHARACTER*16 dum
      CHARACTER*16 dummy
      CHARACTER*16 header(150)
      CHARACTER*1 yorn
      CHARACTER*80 afile
      CHARACTER*80 pfile
!
      INTEGER i
      INTEGER idbnm
      INTEGER ihead1
      INTEGER ihead2
      INTEGER iiii
      INTEGER irun
      INTEGER iset
      INTEGER itogo
      INTEGER ivrmap(50)
      INTEGER iword(100,2)
      INTEGER ixx
      INTEGER j
      INTEGER jj0
      INTEGER k
      INTEGER knpmap(50)
      INTEGER ktogo
      INTEGER m
      INTEGER ndxcnt(50)
      INTEGER nnf
      INTEGER nnn
      INTEGER nppts
      INTEGER numiv(50)
      INTEGER numvar
      INTEGER nuwp
      INTEGER nvrmap(50)
      INTEGER nword
      INTEGER io
      INTEGER bpCount(50)
!
      DOUBLE PRECISION aaa(maxabn)
      DOUBLE PRECISION defaultval
      DOUBLE PRECISION dxr
      DOUBLE PRECISION gval
      DOUBLE PRECISION gvlin
      DOUBLE PRECISION rdx(50)
      DOUBLE PRECISION rxf(50)
      DOUBLE PRECISION rxi(50)
      DOUBLE PRECISION vrmax(50)
      DOUBLE PRECISION vrmin(50)
      DOUBLE PRECISION ydiff
      DOUBLE PRECISION value
      DOUBLE PRECISION bpArr(50,maxabn)
!
      LOGICAL ex
      LOGICAL glvxb(50)
      LOGICAL headfg
!
      EQUIVALENCE (aaa(1),acca)
      DATA fmthead/"(' ',40a18)"/
      DATA fmtdata/'(40g18.8)'/
!
      EXTERNAL datconbd
 
!
!     SET INPUT AND OUTPUT FILE UNITS
!
      defaultval=99.d0
      odie_flg=.false.
!     ipcaf now points to the beginning of ABLOCK
      ipcaf=1
      odie_flg=.true.
      nuri=5
      nuwp=10
      nura=15
      numsg=67
      DO nnf=90,94
         OPEN(nnf,status='scratch')
      END DO
!
      OPEN(UNIT=numsg,status='scratch')
      OPEN(UNIT=12,FILE='odie.fmt',STATUS='old',ERR=2)
    1 READ(12,1001,ERR=2,END=2) aline
      CALL word2(aline,nword,iword)
      CALL chadjl (aline)
      CALL chuppr (aline)
!
      IF(aline(iword(1,1):iword(1,2)).EQ.'FORMATH') THEN
         fmthead=aline(iword(2,1):iword(nword,2))
         GOTO 1
      ENDIF
!
      IF(aline(iword(1,1):iword(1,2)).EQ.'FORMATD') THEN
         fmtdata=aline(iword(2,1):iword(nword,2))
         GOTO 1
      ENDIF
!
      GOTO 1
    2 CLOSE(12)
 
!
!        SET DEFAULTS
!
      chotfg=0
!
!          WRITE HEADER
!
      WRITE(nuwo,1002)
!
!          ASK USER FOR INPUT FILE
!
22    WRITE(nuwo,*) 'What Is Your Tabular Input File Name?'
      FLUSH(nuwo)
      CLOSE(nura)
      READ(nuri,1003) afile
!
!      OPEN AFILE FOR USE IN INPDB
!
      INQUIRE (FILE=afile,EXIST=ex)
      IF (.not.ex) THEN
         WRITE(nuwo,*) " "
         WRITE(nuwo,*) trim(afile)," NOT FOUND IN THIS DIRECTORY. TRY AGAIN."
         WRITE(nuwo,*) " "
         GOTO 22
      ENDIF
      icount=0
      OPEN(nura,FILE=afile)
!
      CALL inpdb
!
      WRITE(nuwo,1004) icount
    3 WRITE(nuwo,1005) (dsname(i),i=1,icount)
!
    4 WRITE(nuwo,1006)
      FLUSH(nuwo)
      READ(nuri,1007) dbname
      dbn=dbname
      CALL chlowr(dbn)
      IF(dbn(1:4).EQ.'quit')THEN
         STOP
      ENDIF
      IF(dbn(1:4).EQ.'list') GOTO 3
      IF(dbn(1:4).EQ.'read') GOTO 22
!
!      FIND DATASET NUMBER
!
      DO 5 i=1,icount
         IF(dbname.EQ.dsname(i)) THEN
            idbnm=i
            GOTO 6
         ENDIF
    5 CONTINUE
      WRITE(nuwo,1008) dbname
      GOTO 4
    6 CONTINUE
      numvar=0
      CALL gmpart(idbnm,numvar,nvrmap,vrmin,vrmax,glvxb,numiv,defaultval)
!
      WRITE(nuwo,1009) dbname,numvar
      DO i=1,numvar
         IF (glvxb(i))THEN
            WRITE(nuwo,1018) abname(nvrmap(i))
         ELSE
            WRITE(nuwo,1019) abname(nvrmap(i)),vrmin(i),vrmax(i),numiv(i)
         ENDIF
      END DO
    7 WRITE(nuwo,*) 'Is A Plot File Needed? (Input Y Or N) '
      FLUSH(nuwo)
      READ(nuri,*) yorn
      CALL chlowr(yorn)
!
      IF (yorn.EQ.'y') THEN
         GOTO 8
!
      ELSE IF (yorn.EQ.'n') THEN
         GOTO 4
!
      ELSE
         GOTO 7
      ENDIF
 
    8 nppts=1
!
      DO i=1,numvar
         knpmap(i)=nvrmap(i)
      END DO
!
      itogo=numvar
!
      DO 15 i=1,numvar
         IF (i.EQ.numvar) THEN
            dummy=abname(knpmap(1))
            WRITE(nuwo,1010)
            GOTO 10
         ENDIF
!
    9    WRITE(nuwo,1011) i
         FLUSH(nuwo)
         READ(nuri,1007) dummy
!
!      CHECK TO SEE IF DUMMY IS LEGIT
!
         dum=dummy
         CALL chlowr(dum)
         IF (dum(1:4).EQ.'list') THEN
            WRITE(nuwo,1012) (abname(nvrmap(iiii)),iiii=1,numvar)
            WRITE(nuwo,1013) (abname(ivrmap(k)),k=1,(i-1))
            GOTO 9
         ENDIF
!
   10    DO j=1,numvar
            IF (dummy.EQ.abname(nvrmap(j))) THEN
               ivrmap(i)=nvrmap(j)
!
!       CHECK TO SEE THAT DUMMY IS NOT A REPEATED VARIABLE
!
               DO k=1,(i-1)
                  IF (ivrmap(i).EQ.ivrmap(k))THEN
                     WRITE(nuwo,1014) dummy
                     GOTO 9
                  ENDIF
               ENDDO
!
!         DUMMY is a legit variable - update remaining variables
!
               m=j
               itogo=itogo-1
               ktogo=0
               DO k=1,itogo
                  ktogo=ktogo+1
                  IF (ivrmap(i).NE.knpmap(ktogo)) THEN
                     knpmap(k)=knpmap(ktogo)
                  ELSE
                     ktogo=ktogo+1
                     knpmap(k)=knpmap(ktogo)
                  ENDIF
               END DO
!
               yorn = ' '
               DO WHILE (yorn .NE. '1' .AND. yorn .NE. '2')
                  WRITE(nuwo,1021) dummy
                  FLUSH(nuwo)
                  READ(nuri,*) yorn
                  CALL chlowr(yorn)
               ENDDO
!
               IF (yorn .EQ. '2') THEN
!                 Input a list of actual breakpoints in G10.7 format.
                  WRITE(nuwo,1020) dummy
                  FLUSH(nuwo)
!
                  DO k=1,maxabn
                     READ(nuri,'(G10.7)',ADVANCE='NO',IOSTAT=io) value
                     IF (io == 0) THEN
                        bpCount(i)=k
                        bpArr(i,k) = value
                     ELSE
!                       Exit loop when the last breakpoint is read in.
                        EXIT
                     ENDIF
                  ENDDO
!
               ELSE
!                 Input min max and increment for breakpoints.
   12             WRITE(nuwo,1015) dummy
                  FLUSH(nuwo)
!
                  READ(nuri,*,IOSTAT=io) rxi(i),rxf(i),rdx(i)
                  IF (io .NE. 0) THEN
                     WRITE(nuwo,*) 'OH NO! AN INPUT ERROR! TRY AGAIN.'
                     GOTO 12
                  ENDIF
                  IF (rxi(i).EQ.0 .AND. rxf(i).EQ.0 .AND.
     +            rdx(i).NE.0 .AND. vrmin(m).NE.defaultval .AND.
     +            vrmax(m).NE.-defaultval) THEN
                     rxi(i)=vrmin(m)
                     rxf(i)=vrmax(m)
                  ENDIF
!
                  dxr=rxf(i)-rxi(i)
                  bpCount(i)=1
!
                  IF (dxr.NE.0 .AND. rdx(i).NE.0) THEN
                     bpCount(i)=(dxr/rdx(i)) + 1.d0
                  ELSE
                     dxr=1.d0
                  ENDIF
                  
                  DO k=1,bpCount(i)
                     bpArr(i,k) = rxi(i) + rdx(i)*(k-1)
                  ENDDO
               ENDIF
!              
               nppts=nppts*bpCount(i)
               GOTO 15
            ENDIF
         ENDDO
         WRITE(nuwo,1016) dummy
         GOTO 9
   15 CONTINUE
!
!      BIGTIME LET'S MAKE A PLOT FILE
!
      IF (numvar.EQ.0) GOTO 4
      WRITE(nuwo,*) 'Please Input Your Plot File Name'
      FLUSH(nuwo)
      READ(nuri,1007) pfile
      OPEN(nuwp,FILE=pfile)
!
!      MAKE UP HEADER
!
      ihead1=0
      DO ixx=1,numvar
         ihead1=ihead1+1
         header(ihead1)=abname(ivrmap(ixx))
      ENDDO
      ihead2=ihead1+1
      header(ihead2)=dsname(idbnm)
      ihead2=ihead2+1
      header(ihead2)='LIN_'//dsname(idbnm)(1:5)
      ihead2=ihead2+1
      header(ihead2)='SPL-LIN'
!
!     WRITE HEADER INFORMATION
!
      irun=1
      iset=1
      WRITE(nuwp,fmthead) (header(jj0),jj0=1,ihead1), (header(jj0),jj0
     +=(ihead1+1),ihead2)
!     WRITE HEADER TITLES
!
      irun=1
      iset=1
      DO k=1,numvar
         ndxcnt(k)=1
      ENDDO
!
!    THIS IS THE BIG LOOP
!
      DO nnn=1,nppts
!
!     LOAD VALUES INTO ARRAYS
!
         DO k=1,numvar
            aaa(ivrmap(k))=bpArr(k,ndxcnt(k))
         ENDDO
!
         CALL gmevl(idbnm,gval)
         CALL lgmevl(idbnm,gvlin)
!
         ydiff=gval-gvlin
         WRITE(nuwp,fmtdata)(aaa(ivrmap(k)),k=1,numvar),gval,gvlin,ydiff
 
!
         IF (nnn.EQ.nppts) THEN
            WRITE(nuwp,1017)
            CYCLE
         ENDIF
!
         headfg=.false.
!
         DO k=1,numvar
            ndxcnt(k)=ndxcnt(k)+1
            IF (ndxcnt(k).LE.bpCount(k)) EXIT
            IF (bpCount(k).GT.1) headfg=.true.
            IF (k.EQ.1) THEN
               irun=irun+1
            ELSEIF (k.EQ.2) THEN
               irun=1
               iset=iset+1
            ENDIF
            ndxcnt(k)=1
         ENDDO
         IF (headfg) THEN
            WRITE(nuwp,1017)
         ENDIF
      ENDDO
!
!      CLOSE NUWP AND GO FOR NEXT DATASET
!
      CLOSE(nuwp)
!
      GOTO 4
!
 1001 FORMAT(132a)
 1002 FORMAT(//,10x,'Otis', / ,11x,'Data', / ,12x,'Interactive', / ,13x,
     +'Examiner',//)
 1003 FORMAT(a80)
 1004 FORMAT(/// ,10x,'This File Contains ',i2, ' Dataset(s)')
 1005 FORMAT(/ ,10x,'The Dataset(S) Is/Are ',/, (/,10x,4a18))
 1006 FORMAT(/ ,'What Dataset Do You Want To Examine?',
     +' (Type Quit To Exit, List To Redisplay The List Of Datasets, Or Read to read a new file)')
 1007 FORMAT(a16)
 1008 FORMAT(/ ,'NO MATCH WAS FOUND FOR THE INPUT ', 'DATASET NAME ',
     +a16)
 1009 FORMAT(/ ,'Dataset ',a16,'Is A Function Of ',i2, ' Thing(s)', / ,
     +'Variable',16x,'Min',16x,'Max',16x,'No. Of Elements')
 1010 FORMAT(a1)
 1011 FORMAT(/ ,'What Is The Number ',i2, ' Thing To Vary?')
 1012 FORMAT(/ ,'Valid Inputs Are ',(/ ,4a16))
 1013 FORMAT(/ ,'These Things Have Been Picked', (/ ,4a16))
 1014 FORMAT(' ',a16,' HAS ALREADY BEEN PICKED  TRY AGAIN')
 1015 FORMAT('Input Start, Stop And Increment Values For ',a)
 1016 FORMAT(' ',a16,' IT NOT A VALID CHOICE  TRY AGAIN')
 1017 FORMAT(' ')
 1018 FORMAT('  ',a16,5x,9('-'),9x,9('-'))
 1019 FORMAT('  ',a16,1x,g17.5,1x,g17.5,1x,i17)
 1020 FORMAT('Input Breakpoint Values in G10.7 Format For ',a)
 1021 FORMAT('Input min/max/incr (1) or list of breakpoints (2) for ',a)

      END
